/* ͻ
	   Vector.cpp        Versin 1.2, Octubre 1.995  
	                                                 
	   Implementacin de la clase Vector.            
	                                                 
	   (C) Alvaro Silgado Herrero.                   
	 ͼ */

#include <math.h>

//#include "rt.hpp"
#include "Vector.hpp"
//#include "Matrix33.hpp"
/* Ŀ
	   Construccin de un Vector por defecto.        
	  */

Vector :: Vector (void)
{
	 x = 0;
	 y = 0;
	 z = 0;
}

/* Ŀ
	   Construcccin de un Vector a partir de sus    
	   coordenadas.                                  
	  */

Vector :: Vector (float cx, float cy, float cz)
{
	 x = cx;
	 y = cy;
	 z = cz;
}

/* Ŀ
	   Sobrecarga del operador igualdad.             
	  */

Vector Vector :: operator = (Vector & v)
{
	 x = v.x;
	 y = v.y;
	 z = v.z;
   return (* this);
}

Vector Vector :: operator = (float n)
{
	 x = n;
	 y = n;
	 z = n;
   return (* this);
}

/* Ŀ
	   Sobrecarga del operador suma, con un Vector.  
    */

Vector Vector :: operator + (Vector & v)
{
	 Vector aux;

   aux.x = x + v.x;
   aux.y = y + v.y;
   aux.z = z + v.z;
   return aux;
}

/* Ŀ
	   Sobrecarga del operador suma, con un float.    
	  */

Vector Vector :: operator + (float n)
{
	 Vector aux;

   aux.x = x + n;
	 aux.y = y + n;
   aux.z = z + n;
   return aux;
}

/* Ŀ
     Sobrecarga del operador resta, unario.        
    */

Vector Vector :: operator - ()
{
	 Vector aux;

	 aux.x = - x;
   aux.y = - y;
   aux.z = - z;
   return aux;
}

/* Ŀ
	   Sobrecarga del operador resta, con un Vector. 
    */

Vector Vector :: operator - (Vector & v)
{
	 Vector aux;

   aux.x = x - v.x;
   aux.y = y - v.y;
   aux.z = z - v.z;
   return aux;
}

/* Ŀ
	   Sobrecarga del operador resta, con un float.   
    */

Vector Vector :: operator - (float n)
{
	 Vector aux;

   aux.x = x - n;
   aux.y = y - n;
   aux.z = z - n;
   return aux;
}

/* Ŀ
     Sobrecarga del operador multiplicacin, con   
	   un Vector.                                    
    */

Vector Vector :: operator * (Vector & v)
{
	 Vector aux;

   aux.x = x * v.x;
   aux.y = y * v.y;
   aux.z = z * v.z;
   return aux;
}

/* Ŀ
     Sobrecarga del operador multiplicacin, con   
	   un float.                                     
	  */

Vector Vector :: operator * (float n)
{
	 Vector aux;

	 aux.x = x * n;
	 aux.y = y * n;
	 aux.z = z * n;
	 return aux;
}

/* Ŀ
	   Sobrecarga del operador divisin, con un      
	   float.                                         
	  */

Vector Vector :: operator / (float n)
{
	 Vector aux;

	 aux.x = x / n;
	 aux.y = y / n;
	 aux.z = z / n;
	 return aux;
}


/* Ŀ
	   Sobrecarga del operador divisin, con un      
	   Vector.  BY MGG                               
	  */

Vector Vector :: operator / (Vector & v)
{
	 Vector aux;

	 aux.x = x / v.x;
	 aux.y = y / v.y;
	 aux.z = z / v.z;
	 return aux;
}

/* Ŀ
	   Sobrecarga del operador mayor que, que pasa   
	   a significar 'algn componente mayor que'.    
	  */

int Vector :: operator > (float r)
{
	 return ((x > r) || (y > r) || (z > r));
}

/* Ŀ
	   Sobrecarga del operador or, que pasa a sig-   
	   nificar 'producto escalar'.                   
	  */

float Vector :: operator | (Vector & v)
{
	 float aux;

	 aux = x * v.x + y * v.y + z * v.z;
	 return aux;
}

/* Ŀ
	   Sobrecarga del operador and, que pasa a sig-  
	   nificar 'producto Vectorial'.                 
	  */

Vector Vector :: operator & (Vector & v)
{
	 Vector aux;

	 aux.x = y * v.z - z * v.y;
	 aux.y = z * v.x - x * v.z;
	 aux.z = x * v.y - y * v.x;
	 return aux;
}

/* Ŀ
	   Sobrecarga del operador not, que pasa a sig-  
	   nificar 'normalizar Vector'.                  
	  */

Vector Vector :: operator ! ()
{
	 Vector aux;
	 float modulo;

	 /* El modulo del Vector es la raiz cuadrada
			de su producto escalar */

	 modulo = sqrt (* this | * this);
	 aux = * this;

	 /* Si el mdulo no es aproximadamente 1 y
			es mayor que 0, divide */

	 if ((modulo > 1 + EPSILON) ||
			((modulo < 1 - EPSILON) && (modulo > EPSILON)))
	 {
		 aux.x = x / modulo;
		 aux.y = y / modulo;
		 aux.z = z / modulo;
	 }

	 return aux;
}

/* Ŀ
	   Hace que ningn componente sea mayor que el   
	   valor introducido.                            
	  */

void Vector :: como_mucho (float max)
{
	 if (x > max)
			x = max;
	 if (x < -max)
			x = -max;

	 if (y > max)
			y = max;
	 if (y < -max)
			y = -max;

	 if (z > max)
			z = max;
	 if (z < -max)
			z = -max;
}

/* Ŀ
	   Devuelve la coordenada dominante del Vector   
	   (la que tiene mayor valor absoluto).          
	  */

int Vector :: dominante ()
{
	 float aux1, aux2;
	 int  i;

	 aux1 = ABS(x);
	 aux2 = ABS(y);
	 if (aux1 > aux2)
	 {
			if (aux1 > ABS(z))
	 i = 0;
			else
	 i = 2;
	 }
	 else
	 {
			if (aux2 > ABS(z))
	 i = 1;
			else
	 i = 2;
	 }

	 return i;
}

/* Ŀ
	   Comprueba si hay alguna componente positiva.  
	  */
																												/* Nuevo */
int Vector :: alguno_positivo ()
{
	 return ((x > EPSILON) || (y > EPSILON) || (z > EPSILON));
}

float Vector::mod(void) {
	como_mucho(500000);
	return sqrt (* this | * this);

}




void Vector :: normaliza(void) {
	*this = *this / mod();
}


int Vector :: operator == (const Vector& v) {
	return (v.x==x && v.y==y && v.z==z);
	//return (ABS(v.x-x)<EPSILON && ABS(v.y-y)<EPSILON && ABS(v.z-z)<EPSILON);
}
